/*
 * Copyright (C) 2000  Gediminas (gugini@fortas.ktu.lt)
 * Copyright (C) 2002  Bartomiej Butyn (bartek@milc.com.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#define IPBAN_INTERNAL_ACCESS
#include "common/setup_before.h"
#include <stdio.h>
#ifdef HAVE_STDDEF_H
# include <stddef.h>
#else
# ifndef NULL
#  define NULL ((void *)0)
# endif
#endif
#ifdef STDC_HEADERS
# include <stdlib.h>
#else
# ifdef HAVE_MALLOC_H
#  include <malloc.h>
# endif
#endif
#ifdef HAVE_STRING_H
# include <string.h>
#else
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
#endif
#include "compat/strchr.h"
#include "compat/strdup.h"
#include "compat/strcasecmp.h"
#include <errno.h>
#include "compat/strerror.h"
#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#include "common/eventlog.h"
#include "common/util.h"
#include "connection.h"
#include "message.h"
#include "prefs.h"
#include "common/list.h"

#include "ipban.h"
#include "common/setup_after.h"


static int ipbanlist_unload_entry(t_ipban_entry * e);
static unsigned long ipban_str2ulong(char const * ipaddr);
static int ipbanlist_identical_entry(t_ipban_entry * e1, t_ipban_entry * e2);
static t_ipban_entry * ipban_str_to_ipban_entry(char const * cp);
static int ipbanlist_check(t_connection * c, char const * cp);
static void ipban_usage(t_connection * c);
static int ipbanlist_list(t_connection * c);
static int identify_ipban_function(const char * funcstr);

static t_list * ipbanlist_head=NULL;
static t_ipban_entry * 	base=NULL;
static time_t		lastchecktime = 0;


extern int ipban_check(char const * ipaddr)
{
    char *          whole;
    char const *    ip1;
    char const *    ip2;
    char const *    ip3;
    char const *    ip4;
    t_ipban_entry * temp;
    int		    counter;
    time_t	    now;
    
    if (!ipaddr)
    {
	eventlog(eventlog_level_warn,"ipban_check","got NULL ipaddr");
	return -1;
    }
    
    if (!(whole = strdup(ipaddr)))
    {
    	eventlog(eventlog_level_warn,"ipban_check","could not allocate memory to check ip against wildcard");
	return -1;
    }
    
    time(&now);
    eventlog(eventlog_level_debug,"ipban_check","lastcheck: %u, now: %u, now-lc: %u.",(unsigned)lastchecktime,(unsigned)now,(unsigned)(now-lastchecktime));
    
    if (now - lastchecktime >= prefs_get_ipban_check_int()) /* unsigned; no need to check prefs < 0 */
    {
	ipbanlist_unload_expired();
	lastchecktime = now;
    }
    
    ip1 = strtok(whole,".");
    ip2 = strtok(NULL,".");
    ip3 = strtok(NULL,".");
    ip4 = strtok(NULL,".");
    
    if (!ip1 || !ip2 || !ip3 || !ip4)
    {
	eventlog(eventlog_level_warn,"ipban_check","got bad IP address \"%s\"",ipaddr);
	free(whole);
	return -1;
    }
    
    eventlog(eventlog_level_debug,"ipban_check","checking %s.%s.%s.%s",ip1,ip2,ip3,ip4);
    
    for (temp=base,counter=1; temp; temp=temp->next,counter++)
	switch (temp->type)
	{
	case ipban_type_exact:
	    if (strcmp(temp->info1,ipaddr)==0)
	    {
		eventlog(eventlog_level_debug,"ipban_check","address %s matched exact %s",ipaddr,temp->info1);
		free(whole);
		return counter;
	    }
	    eventlog(eventlog_level_debug,"ipban_check","address %s does not match exact %s",ipaddr,temp->info1);
	    continue;
	 
	case ipban_type_wildcard:
	    if (strcmp(temp->info1,"*")!=0 && strcmp(ip1,temp->info1)!=0)
	    {
		eventlog(eventlog_level_debug,"ipban_check","address %s does not match part 1 of wildcard %s.%s.%s.%s",ipaddr,temp->info1,temp->info2,temp->info3,temp->info4);
		continue;
	    }
	    if (strcmp(temp->info2,"*")!=0 && strcmp(ip2,temp->info2)!=0)
	    {
		eventlog(eventlog_level_debug,"ipban_check","address %s does not match part 2 of wildcard %s.%s.%s.%s",ipaddr,temp->info1,temp->info2,temp->info3,temp->info4);
		continue;
	    }
	    if (strcmp(temp->info3,"*")!=0 && strcmp(ip3,temp->info3)!=0)
	    {
		eventlog(eventlog_level_debug,"ipban_check","address %s does not match part 3 of wildcard %s.%s.%s.%s",ipaddr,temp->info1,temp->info2,temp->info3,temp->info4);
		continue;
	    }
	    if (strcmp(temp->info4,"*")!=0 && strcmp(ip4,temp->info4)!=0)
	    {
		eventlog(eventlog_level_debug,"ipban_check","address %s does not match part 4 of wildcard %s.%s.%s.%s",ipaddr,temp->info1,temp->info2,temp->info3,temp->info4);
		continue;
	    }
	    
	    eventlog(eventlog_level_debug,"ipban_check","address %s matched wildcard %s.%s.%s.%s",ipaddr,temp->info1,temp->info2,temp->info3,temp->info4);
	    free(whole);
	    return counter;
	    
	case ipban_type_range:
	    if ((ipban_str2ulong(ipaddr) >= ipban_str2ulong(temp->info1)) &&
		(ipban_str2ulong(ipaddr) <= ipban_str2ulong(temp->info2)))
	    {
		eventlog(eventlog_level_debug,"ipban_check","address %s matched range %s-%s",ipaddr,temp->info1,temp->info2);
		free(whole);
		return counter;
	    }
	    eventlog(eventlog_level_debug,"ipban_check","address %s does not match range %s-%s",ipaddr,temp->info1,temp->info2);
	    continue;
	
	case ipban_type_netmask:
	    {
		unsigned long	lip1;
		unsigned long	lip2;
		unsigned long	netmask;

		lip1 = ipban_str2ulong(ipaddr);
		if (!lip1) return -1;
		lip2 = ipban_str2ulong(temp->info1);
		if (!lip2) return -1;
		netmask = ipban_str2ulong(temp->info2);
		if (!netmask) return -1;

		lip1 = lip1 & netmask;
		lip2 = lip2 & netmask;		
		if (lip1 == lip2)
		{
		    eventlog(eventlog_level_debug,"ipban_check","address %s matched netmask %s/%s",ipaddr,temp->info1,temp->info2);
		    free(whole);
		    return counter;
		}
		eventlog(eventlog_level_debug,"ipban_check","address %s does not match netmask %s/%s",ipaddr,temp->info1,temp->info2);
		continue;
	    }	
	
	case ipban_type_prefix:
	    {
		unsigned long	lip1;
		unsigned long	lip2;
		int		prefix;
				
		lip1 = ipban_str2ulong(ipaddr);
		if (!lip1) return -1;
		lip2 = ipban_str2ulong(temp->info1);
		if (!lip2) return -1;
		prefix = atoi(temp->info2);

		lip1 = lip1 >> (32 - prefix);
		lip2 = lip2 >> (32 - prefix);		
		if (lip1 == lip2)
		{
		    eventlog(eventlog_level_debug,"ipban_check","address %s matched prefix %s/%s",ipaddr,temp->info1,temp->info2);
		    free(whole);
		    return counter;
		}
		eventlog(eventlog_level_debug,"ipban_check","address %s does not match prefix %s/%s",ipaddr,temp->info1,temp->info2);
		continue;
	    }	
	default:  /* unknown type */
	    eventlog(eventlog_level_warn,"ipban_check","found bad ban type %d",(int)temp->type);
	}
    
    free(whole);
    return 0;
}


extern int ipbanlist_load(char const * filename)
{
    FILE *          fp;
    char *          buff;
    char *          cp;
    unsigned int    currline;
    
    base = NULL; /* FIXME wtf? what if ipbanlist isn't empty? Memory leak. And if ipbanlist is empty - base point to NULL so it is not necesary. I think that better will be: if base != NULL - error, so we check that list is empty, before reading it */
    
    if (!filename)
    {
        eventlog(eventlog_level_error,"ipbanlist_load","got NULL filename");
	return -1;
    }
    
    if (!(fp = fopen(filename,"r")))
    {
        eventlog(eventlog_level_error,"ipbanlist_load","could not open banlist file \"%s\" for reading (fopen: %s)",filename,strerror(errno));
	return -1;
    }
    
    for (currline=1; (buff = file_get_line(fp)); currline++)
    {
	cp = buff;
	
	/* eat whitespace in front */
	while (*cp=='\t' || *cp==' ') cp++;
	if (*cp=='\0' || *cp=='#')
	{
	    free(buff);
	    continue;
	}
	
	/* eat whitespace in back */
	while (strlen(cp)>0 && (cp[strlen(cp)-1]==' ' || cp[strlen(cp)-1]=='\t'))
	    cp[strlen(cp)-1] = '\0';
	
	if (ipbanlist_add(NULL,cp,"")!=0)
	{
	    free(buff);
	    continue;
	}
	
	free(buff);
    }
    
    if (fclose(fp)<0)
        eventlog(eventlog_level_error,"ipbanlist_load","could not close banlist file \"%s\" after reading (fclose: %s)",filename,strerror(errno));
    
    return 0;
}


static int ipbanlist_unload_entry(t_ipban_entry * e)
{
    eventlog(eventlog_level_warn,"ipbanlist_unload","unloading type: %d",(int)e->type);
    switch (e->type)
    {
	case ipban_type_exact:
	case ipban_type_wildcard:
	    if (e->info1)
		free(e->info1);
	    break;
	case ipban_type_range:
	case ipban_type_netmask:
	case ipban_type_prefix:
	    if (e->info1)
		free(e->info1);
	    if (e->info2)
		free(e->info2);
	    break;
	default:  /* unknown type */
    	    eventlog(eventlog_level_warn,"ipbanlist_unload","found bad ban type %d",(int)e->type);
	    return -1;
    }
    free(e);
    return 0;
}


extern int ipbanlist_unload(void)
{
    t_ipban_entry * temp;
    t_ipban_entry * next;

    for (temp=base; temp; temp=next)
    {
	ipbanlist_unload_entry(temp);
	if (base==temp)
	   base = next;
    }
    
    return 0;
}


extern int ipbanlist_unload_expired(void)
{
    t_ipban_entry * 	curr;
    t_ipban_entry * 	prev;
    time_t		now;
    
    eventlog(eventlog_level_debug,"ipbanlist_unload_expired","checking for expired bans.");

    time(&now);
    curr = base;
    if (!curr) return 0;
    prev = curr;
    do
    {
	if ((curr->endtime - now <= 0) && (curr->endtime != 0))
	{
	    eventlog(eventlog_level_debug,"ipbanlist_unload_expired","deleting: %s",curr->info1);
	    prev->next = curr->next;
	    ipbanlist_unload_entry(curr);
	    if (base==curr)
		base = prev->next;
	    curr = prev->next;
	}
	else
	{
	    prev = curr;
	    curr = curr->next;
	}
    } while (curr != NULL);

    return 0;
}


static unsigned long ipban_str2ulong(char const * ipaddr)
{
    unsigned long	lip;
    char *    		ip1;
    char *    		ip2;
    char *    		ip3;
    char *    		ip4;
    char *		tipaddr;
    
    if (!(tipaddr = strdup(ipaddr)))
    {
        eventlog(eventlog_level_warn,"ipban_ipstr2ulong","could not allocate memory for tipaddr");
	return 0;
    }
    ip1 = strtok(tipaddr,".");
    ip2 = strtok(NULL,".");
    ip3 = strtok(NULL,".");
    ip4 = strtok(NULL,".");
    lip = (atoi(ip1) << 24) + (atoi(ip2) << 16) + (atoi(ip3) << 8) + (atoi(ip4));

    free(tipaddr);
    return lip;
}


static int ipbanlist_identical_entry(t_ipban_entry * e1, t_ipban_entry * e2)
{
    switch (e2->type)
    {
	case ipban_type_exact:
	    if (strcmp(e1->info1,e2->info1)==0)
		return 1;
	    break;
	case ipban_type_range:
	    if (strcmp(e1->info1,e2->info1)==0 &&
		strcmp(e1->info2,e2->info2)==0)
		return 1;
	    break;
	case ipban_type_wildcard:
	    if (strcmp(e1->info1,e2->info1)==0 &&
		strcmp(e1->info2,e2->info2)==0 &&
		strcmp(e1->info3,e2->info3)==0 &&
		strcmp(e1->info4,e2->info4)==0)
		return 1;
	    break;
	case ipban_type_netmask:
	    if (strcmp(e1->info1,e2->info1)==0 &&
		strcmp(e1->info2,e2->info2)==0)
		return 1;
	    break;
	case ipban_type_prefix:
	    if (strcmp(e1->info1,e2->info1)==0 &&
		strcmp(e1->info2,e2->info2)==0)
		return 1;
	    break;
        default:  /* unknown type */
	    eventlog(eventlog_level_warn,"ipbanlist_identical_entry","found bad ban type %d",(int)e2->type);
    }
    return 0;
}


static t_ipban_entry * ipban_str_to_ipban_entry(char const * cp)
{
    char *          matched;
    char *          whole;
    t_ipban_entry * temp;

    if (!(temp = malloc(sizeof(t_ipban_entry))))
    {
        eventlog(eventlog_level_error,"ipban_str_to_t_ipban_entry","could not allocate memory for entry %s",cp);
        return NULL;
    }

    if (cp[0] == '\0')
    {
        eventlog(eventlog_level_debug,"ipban_str_to_t_ipban_entry","got NULL IP.");
        return NULL;
    }
    
    if ((matched = strchr(cp,'-'))) /* range */
    {
        temp->type = ipban_type_range;
	eventlog(eventlog_level_debug,"ipban_str_to_ipban_entry","entry: %s matched as ipban_type_range",cp);
        matched[0] = '\0';
        if (!(temp->info1 = strdup(cp))) /* start of range */
        {
	    eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry %s",cp);
	    free(temp);
	    return NULL;
	}
	if (!(temp->info2 = strdup(&matched[1]))) /* end of range */
	{
	    eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry %s",cp);
	    free(temp->info1);
	    free(temp);
	    return NULL;
	}
	temp->info3 = NULL; /* clear unused elements so debugging is nicer */
	temp->info4 = NULL;
    }
    else
        if (strchr(cp,'*')) /* wildcard */
        {
	    temp->type = ipban_type_wildcard;
	    eventlog(eventlog_level_debug,"ipban_str_to_ipban_entry","entry: %s matched as ipban_type_wildcard",cp);
		
	    /* only free() info1! */
	    if (!(whole = strdup(cp)))
	    {
	        eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry \"%s\"",cp);
	        free(temp);
	        return NULL;
	    }
	    temp->info1 = strtok(whole,".");
	    temp->info2 = strtok(NULL,".");
	    temp->info3 = strtok(NULL,".");
	    temp->info4 = strtok(NULL,".");
	    if (!temp->info4) /* not enough dots */
	    {
		eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","wildcard entry \"%s\" does not contain all four octets",cp);
		free(temp->info1);
		free(temp);
		return NULL;
	    }
	}
	else
	    if ((matched = strchr(cp,'/'))) /* netmask or prefix */
	    {
		if (strchr(&matched[1],'.'))
		{
		    temp->type = ipban_type_netmask;
		    eventlog(eventlog_level_debug,"ipban_str_to_ipban_entry","entry: %s matched as ipban_type_netmask",cp);
		}
		else
		{
		    temp->type = ipban_type_prefix;
		    eventlog(eventlog_level_debug,"ipban_str_to_ipban_entry","entry: %s matched as ipban_type_prefix",cp);
		}
	    
		matched[0]='\0';
		if (!(temp->info1 = strdup(cp)))
		{
		    eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry \"%s\"",cp);
	    	    free(temp);
		    return NULL;
		}
		if (!(temp->info2 = strdup(&matched[1])))
		{
		    eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry \"%s\"",cp);
		    free(temp->info1);
		    free(temp);
		    return NULL;
		}
		temp->info3 = NULL; /* clear unused elements so debugging is nicer */
		temp->info4 = NULL;
	    }
	    else /* exact */
	    {
		temp->type = ipban_type_exact;
		eventlog(eventlog_level_debug,"ipban_str_to_ipban_entry","entry: %s matched as ipban_type_exact",cp);
	    
		if (!(temp->info1 = strdup(cp)))
		{
	        eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry \"%s\"",cp);
	        free(temp);
	        return NULL;
		}
		temp->info2 = NULL; /* clear unused elements so debugging is nicer */
		temp->info3 = NULL;
		temp->info4 = NULL;
	    }

    return temp;
}


extern int ipbanlist_add(t_connection * c, char const * cp, char const * timestr)
{
    t_ipban_entry *	temp;
    time_t		now;
    unsigned int	bmin;
    
    temp = ipban_str_to_ipban_entry(cp);
    if (temp == NULL)
    {
	if (c != NULL)
	    message_send_text(c,message_type_error,c,"Bad IP");
        eventlog(eventlog_level_error,"ipbanlist_add","could not convert to t_ipban_entry: \"%s\"",cp);
        return -1;
    }
    if (clockstr_to_seconds(timestr,&bmin)<0) /* it thinks these are seconds but we treat them as minutes */
    {
	eventlog(eventlog_level_error,"ipbanlist_add","could not convert to minutes: \"%s\"",timestr);
	return -1;

    }

    if (bmin == 0)
    	temp->endtime = 0;
    else
    {	
	time(&now);
	temp->endtime = now + bmin*60;
    }
    temp->next = NULL;
    
    /* add */
    {
	if (base)
	{
	   t_ipban_entry * t;
    
	   for (t=base; t->next; t=t->next);
		t->next = temp;
	}
	else
	    base = temp;
    }

   if (c != NULL)    
     {
	char         tstr[MAX_MESSAGE_LEN];
	char const * tname;
	
	if (bmin == 0)
	  {
	     sprintf(tstr,"%s banned pernamently by %s.",cp,(tname = conn_get_username(c)));
	     conn_unget_username(c,tname);
	     eventlog(eventlog_level_info,"ipbanlist_add",tstr);
	     message_send_admins(c,message_type_info,tstr);
	     sprintf(tstr,"%s banned pernamently.",cp);
	     message_send_text(c,message_type_info,c,tstr);
	  }
	else
	  {
	     sprintf(tstr,"%s banned for %i minutes by %s.",cp,bmin,conn_get_username(c));
	     eventlog(eventlog_level_info,"ipbanlist_add",tstr);
	     message_send_admins(c,message_type_info,tstr);
	     sprintf(tstr,"%s banned for %i minutes.",cp,bmin);
	     message_send_text(c,message_type_info,c,tstr);
	  }
     }
   return 0;
}


extern int ipbanlist_del(t_connection * c, char const * cp)
{
    t_ipban_entry *	to_delete;
    int			to_delete_nmbr;
    t_ipban_entry *	curr;
    t_ipban_entry *	prev;
    unsigned int       	counter;
    char		tstr[MAX_MESSAGE_LEN];
    
    curr = base;
    prev = curr;
    if (!curr)
    {
	message_send_text(c,message_type_error,c,"Empty banlist.");
	return -1;
    }

   if (strchr(cp,'.') || strchr(cp,'/') || strchr(cp,'*'))
    {
	to_delete = ipban_str_to_ipban_entry(cp);
        if (to_delete == NULL)
	{
	    message_send_text(c,message_type_error,c,"Illegal IP entry.");
	    return -1;
	}
	do
	{
	    if (ipbanlist_identical_entry(to_delete,curr))
	    {
	       free(to_delete);
		prev->next = curr->next;
		ipbanlist_unload_entry(curr);
	       if (base==curr)
		 base = prev->next;
		message_send_text(c,message_type_info,c,"Entry deleted.");
		return 0;
	    }
	    prev = curr;
	    curr = curr->next;
	} while (curr != NULL);
       free(to_delete);
       message_send_text(c,message_type_error,c,"No matching entry.");
       return -1;
    }
   
   to_delete_nmbr = atoi(cp);
   if (to_delete_nmbr <= 0)
     {
	message_send_text(c,message_type_error,c,"Wrong entry number.");
	return -1;
     }
   counter = 1;
   do
     {
	if (to_delete_nmbr == counter)
	  {
	     prev->next = curr->next;
	     ipbanlist_unload_entry(curr);
	     if (base==curr)
	       base = prev->next;
	     message_send_text(c,message_type_info,c,"Entry deleted.");
	     return 0;
	  }
	prev = curr;
	curr = curr->next;
	counter++;
     } while (curr != NULL);
	
   if (to_delete_nmbr > counter)
     {
	sprintf(tstr,"There are only %u entries.",counter);
	message_send_text(c,message_type_error,c,tstr);
	return -1;
     }
   return 0;
   
}


static int ipbanlist_check(t_connection * c, char const * cp)
{
    int		res;
    char	temp[MAX_MESSAGE_LEN];
    
    res = ipban_check(cp);
    switch (res)
    {
	case 0:
	    message_send_text(c,message_type_info,c,"IP not banned.");
	    break;
	case -1:
	    message_send_text(c,message_type_error,c,"Error occured.");
	    break;
	default:
	    sprintf(temp,"IP banned by rule #%i.",res);
	    message_send_text(c,message_type_info,c,temp);
    }	
    return 0;
}


static void ipban_usage(t_connection * c)
{
    message_send_text(c,message_type_info,c,"to print this information:");
    message_send_text(c,message_type_info,c,"    /ipban h[elp]");
    message_send_text(c,message_type_info,c,"to print all baned IPs");
    message_send_text(c,message_type_info,c,"    /ipban [l[ist]]");
    message_send_text(c,message_type_info,c,"to erase ban:");
    message_send_text(c,message_type_info,c,"    /ipban d[el] <IP|index num>");
    message_send_text(c,message_type_info,c,"    (IP have to be entry accepted in bnban)");
    message_send_text(c,message_type_info,c,"to add ban:");
    message_send_text(c,message_type_info,c,"    /ipban a[dd] IP");
    message_send_text(c,message_type_info,c,"    (IP have to be entry accepted in bnban)");
    message_send_text(c,message_type_info,c,"to check is specified IP banned:");
    message_send_text(c,message_type_info,c,"    /ipban c[heck] IP");
}


static int ipbanlist_list(t_connection * c)
{
    t_ipban_entry * 	temp;
    char		tstr[MAX_MESSAGE_LEN];
    unsigned int	counter;
    char		timestr[MAX_TIME_STR];
    time_t		now;

    temp = base;
    if (!temp)
    {
	message_send_text(c,message_type_info,c,"Banned IPs: none");
	return 0;
    }
    
    time(&now);
    message_send_text(c,message_type_info,c,"Banned IPs:");
   counter = 1;
    do
    {
	if (temp->endtime == 0)
	    sprintf(timestr,"(perm)");
	else
	{
	    time_t		ttime;
	    unsigned int	hours;
	    unsigned int	minutes;
	    unsigned int	seconds;

	    ttime = temp->endtime - now;
	    hours = ttime / 3600;
	    ttime = ttime % 3600;
	    minutes = ttime / 60;
	    seconds = ttime % 60;
    
	    sprintf(timestr,"(%u:%u:%u left)",hours,minutes,seconds);
	}
	switch (temp->type)
	{
	    case ipban_type_exact:
		sprintf(tstr,"%u: %s %s",counter++,temp->info1,timestr);
		break;
	    case ipban_type_wildcard:
		sprintf(tstr,"%u: %s.%s.%s.%s %s",counter++,temp->info1,temp->info2,temp->info3,temp->info4,timestr);
		break;
	     case ipban_type_range:
		sprintf(tstr,"%u: %s-%s %s",counter++,temp->info1,temp->info2,timestr);
		break;
	     case ipban_type_netmask:
		sprintf(tstr,"%u: %s/%s %s",counter++,temp->info1,temp->info2,timestr);
		break;
	     case ipban_type_prefix:
		sprintf(tstr,"%u: %s/%s %s",counter++,temp->info1,temp->info2,timestr);
		break;
	    
	    default: /* unknown type */
		eventlog(eventlog_level_warn,"ipban_check","found bad ban type %d for line %u",(int)temp->type,counter);
	}
	message_send_text(c,message_type_info,c,tstr);
       counter++;
    } while ((temp = temp->next));
    
    return 0;
}


extern int handle_ipban_command(t_connection * c, char const * text)
{
    char		subcommand[MAX_FUNC_LEN];
    char		ipstr[MAX_IP_STR];
    char		timestr[MAX_TIME_STR];
    unsigned int 	i,j;
    
    for (i=0; text[i]!=' ' && text[i]!='\0'; i++); /* skip command */
    for (; text[i]==' '; i++);
    
    for (j=0; text[i]!=' ' && text[i]!='\0'; i++) /* get subcommand */
	if (j<sizeof(subcommand)-1) subcommand[j++] = text[i];
    subcommand[j] = '\0';
    for (; text[i]==' '; i++);

    for (j=0; text[i]!=' ' && text[i]!='\0'; i++) /* get ip address */
	if (j<sizeof(ipstr)-1) ipstr[j++] = text[i];
    ipstr[j] = '\0';
    for (; text[i]==' '; i++);

    for (j=0; text[i]!=' ' && text[i]!='\0'; i++) /* get time */
	if (j<sizeof(timestr)-1) timestr[j++] = text[i];
    timestr[j] = '\0';
    
    switch (identify_ipban_function(subcommand))
    {
	case IPBAN_FUNC_ADD:
	    ipbanlist_add(c,ipstr,timestr);
	    break;
	case IPBAN_FUNC_DEL:
	    ipbanlist_del(c,ipstr);
	    break;
	case IPBAN_FUNC_LIST:
	    ipbanlist_list(c);
	    break;
	case IPBAN_FUNC_CHECK:
	    ipbanlist_check(c,ipstr);
	    break;
	case IPBAN_FUNC_HELP:
	    message_send_text(c,message_type_info,c,"The ipban command supports the following patterns.");
	    ipban_usage(c);
	    break;
	default:
	    message_send_text(c,message_type_info,c,"The command is inncorect. Use one of the following patterns.");
	    ipban_usage(c);
    }
    return 0;
}
    

static int identify_ipban_function(const char * funcstr)
{
    if (strcasecmp(funcstr,"add")==0 || strcasecmp(funcstr,"a")==0)
	return IPBAN_FUNC_ADD;
    if (strcasecmp(funcstr,"del")==0 || strcasecmp(funcstr,"d")==0)
	return IPBAN_FUNC_DEL;
    if (strcasecmp(funcstr,"list")==0 || strcasecmp(funcstr,"l")==0 || strcmp(funcstr,"")==0)
	return IPBAN_FUNC_LIST;
    if (strcasecmp(funcstr,"check")==0 || strcasecmp(funcstr,"c")==0)
	return IPBAN_FUNC_CHECK;
    if (strcasecmp(funcstr,"help")==0 || strcasecmp(funcstr,"h")==0)
	return IPBAN_FUNC_HELP;
    return IPBAN_FUNC_UNKNOWN;
}
